@@ -281,7 +281,7 @@ class VCloudConnection(ConnectionUserAndKey):
         return {
             'Authorization': "Basic %s" % base64.b64encode(
                 b('%s:%s' % (self.user_id, self.key))).decode('utf-8'),
-            'Content-Length': 0,
+            'Content-Length': '0',
             'Accept': 'application/*+xml'
         }
 
@@ -732,7 +732,7 @@ class HostingComConnection(VCloudConnection):
         return {
             'Authentication': base64.b64encode(b('%s:%s' % (self.user_id,
                                                             self.key))),
-            'Content-Length': 0
+            'Content-Length': '0'
         }
 
 
